// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'document.dart';
import 'index_directory.dart';

class IndexReader {

  final IndexDirectory _directory;

  IndexReader(this._directory);

  Map<String,Map<String,List<int>>> _indexes;
  Map<String,Map<int,String>> _data;

  void read() {
    _indexes = _directory.inputIndex();
    _data = _directory.inputData();
  }

  int docFreq(String field, String text) => _indexes[field][text].length;
  int maxDoc() => _data['id'].length;

  Document document(int docId, Iterable<String> fieldsToLoad) {
    Document ret = new Document();
    for (String field in fieldsToLoad) {
      ret.add(new Field(field, _data[field][docId], null));
    }
    return ret;
  }

  Map<String,List<int>> getTerms(String field) => _indexes[field];
  Map<int,String> getData(String field) => _data[field];

}

class DirectoryReader {
  static IndexReader open(IndexDirectory directory) => new IndexReader(directory)..read();
}